@@ -5,10 +5,10 @@ use strict;
 use Mouse;
 use JSON qw(from_json to_json);
 
-our $VERSION = '2.0.14';
+our $VERSION = '2.0.15';
 
 extends qw(
-  Lemonldap::NG::Portal::Main::Plugin
+  Lemonldap::NG::Portal::2F::Register::Base
   Lemonldap::NG::Common::TOTP
 );
 
@@ -18,7 +18,7 @@ has prefix   => ( is => 'rw', default => 'totp' );
 has template => ( is => 'ro', default => 'totp2fregister' );
 has welcome  => ( is => 'ro', default => 'yourNewTotpKey' );
 has logo     => ( is => 'rw', default => 'totp.png' );
-has ott      => (
+has ott => (
     is      => 'rw',
     lazy    => 1,
     default => sub {
@@ -31,10 +31,6 @@ has ott      => (
     }
 );
 
-sub init {
-    return 1;
-}
-
 sub run {
     my ( $self, $req, $action ) = @_;
     my $user = $req->userData->{ $self->conf->{whatToTrace} };
