@@ -8,35 +8,33 @@ use strict;
 use Mouse;
 use JSON qw(from_json to_json);
 use Lemonldap::NG::Portal::Main::Constants qw(
-  PE_BADOTP
-  PE_ERROR
-  PE_FORMEMPTY
   PE_OK
+  PE_ERROR
+  PE_BADOTP
+  PE_FORMEMPTY
   PE_SENDRESPONSE
 );
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.0.15';
 
-extends 'Lemonldap::NG::Portal::Main::SecondFactor',
-  'Lemonldap::NG::Common::TOTP';
+extends qw(
+  Lemonldap::NG::Portal::Main::SecondFactor
+  Lemonldap::NG::Common::TOTP
+);
 
 # INITIALIZATION
 
 has prefix => ( is => 'ro', default => 'totp' );
-
-has logo => ( is => 'rw', default => 'totp.png' );
+has logo   => ( is => 'rw', default => 'totp.png' );
 
 sub init {
     my ($self) = @_;
 
-    # If self registration is enabled and "activation" is just set to
-    # "enabled", replace the rule to detect if user has registered its key
-    if (    $self->conf->{totp2fSelfRegistration}
-        and $self->conf->{totp2fActivation} eq '1' )
-    {
-        $self->conf->{totp2fActivation} =
-          '$_2fDevices && $_2fDevices =~ /"type":\s*"TOTP"/s';
-    }
+    # If "activation" is just set to "enabled",
+    # replace the rule to detect if user has registered its key
+    $self->conf->{totp2fActivation} = 'has2f("TOTP")'
+      if $self->conf->{totp2fActivation} eq '1';
+
     return $self->SUPER::init();
 }
 
@@ -47,17 +45,21 @@ sub run {
     $self->logger->debug('Generate TOTP form');
 
     my $checkLogins = $req->param('checkLogins');
-    $self->logger->debug("TOTP checkLogins set") if ($checkLogins);
+    $self->logger->debug("TOTP: checkLogins set") if $checkLogins;
+
+    my $stayconnected = $req->param('stayconnected');
+    $self->logger->debug("TOTP: stayconnected set") if $stayconnected;
 
     # Prepare form
     my $tmp = $self->p->sendHtml(
         $req,
         'totp2fcheck',
         params => {
-            MAIN_LOGO   => $self->conf->{portalMainLogo},
-            SKIN        => $self->p->getSkin($req),
-            TOKEN       => $token,
-            CHECKLOGINS => $checkLogins
+            MAIN_LOGO     => $self->conf->{portalMainLogo},
+            SKIN          => $self->p->getSkin($req),
+            TOKEN         => $token,
+            CHECKLOGINS   => $checkLogins,
+            STAYCONNECTED => $stayconnected
         }
     );
     $self->logger->debug("Prepare TOTP 2F verification");
