@@ -12,9 +12,10 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
   PE_BADURL
   PE_SENDRESPONSE
+  URIRE
 );
 
-our $VERSION = '2.0.9';
+our $VERSION = '2.0.14';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::CAS';
@@ -59,10 +60,13 @@ sub init {
     );
 
     # Add CAS Services, so we can check service= parameter on logout
-    foreach my $casSrv ( keys %{ $self->conf->{casAppMetaDataOptions} } ) {
-        if ( my $serviceUrl =
-            $self->conf->{casAppMetaDataOptions}->{$casSrv}
-            ->{casAppMetaDataOptionsService} )
+    foreach my $casSrv ( keys %{ $self->casAppList } ) {
+        for my $serviceUrl (
+            split(
+                /\s+/,
+                $self->casAppList->{$casSrv}->{casAppMetaDataOptionsService}
+            )
+          )
         {
             push @{ $self->p->{additionalTrustedDomains} }, $serviceUrl;
             $self->logger->debug(
@@ -83,27 +87,33 @@ sub storeEnvAndCheckGateway {
       || $req->param('gateway');
 
     if ( $gateway and $gateway eq "true" ) {
-        $self->logger->debug(
-            "Gateway mode requested, redirect without authentication");
-        $req->response( [ 302, [ Location => $service ], [] ] );
-        for my $s ( $self->ipath, $self->ipath . 'Path' ) {
-            $self->logger->debug("Removing $s from pdata")
-              if delete $req->pdata->{$s};
-        }
+        if ( $self->_gatewayAllowedRedirect( $req, $service ) ) {
+            $self->logger->debug(
+                "Gateway mode requested, redirect without authentication");
+            $req->response( [ 302, [ Location => $service ], [] ] );
+            for my $s ( $self->ipath, $self->ipath . 'Path' ) {
+                $self->logger->debug("Removing $s from pdata")
+                  if delete $req->pdata->{$s};
+            }
 
-        return PE_SENDRESPONSE;
+            return PE_SENDRESPONSE;
+        }
+        else {
+            $self->logger->error(
+                "Disallowing redirection to unknown service $service");
+            return PE_CAS_SERVICE_NOT_ALLOWED;
+        }
     }
 
-    if ( $service and $service =~ m#^(https?://[^/]+)(/.*)?$# ) {
-        my ( $host, $uri ) = ( $1, $2 );
-        my $app = $self->casAppList->{$host};
+    if ( $service and $service =~ URIRE ) {
+        my $app = $self->getCasApp($service);
 
         if ($app) {
             $req->env->{llng_cas_app} = $app;
 
             # Store target authentication level in pdata
-            my $targetAuthnLevel = $self->conf->{casAppMetaDataOptions}->{$app}
-              ->{casAppMetaDataOptionsAuthnLevel};
+            my $targetAuthnLevel =
+              $self->casAppList->{$app}->{casAppMetaDataOptionsAuthnLevel};
             $req->pdata->{targetAuthnLevel} = $targetAuthnLevel
               if $targetAuthnLevel;
 
@@ -113,6 +123,24 @@ sub storeEnvAndCheckGateway {
     return PE_OK;
 }
 
+sub _gatewayAllowedRedirect {
+    my ( $self, $req, $service ) = @_;
+
+    my $app                    = $self->getCasApp($service);
+    my $casAccessControlPolicy = $self->conf->{casAccessControlPolicy};
+
+    # Redirect is allowed if there is no access control or if
+    # the service is declared in CAS apps
+    if ( $casAccessControlPolicy !~ /^(error|faketicket)$/i ) {
+        return 1;
+    }
+    if ($app) {
+        return 1;
+    }
+
+    return 0;
+}
+
 # Main method (launched only for authenticated users, see Main/Issuer)
 sub run {
     my ( $self, $req, $target ) = @_;
@@ -139,22 +167,27 @@ sub run {
     # Session ID
     my $session_id = $req->{sessionInfo}->{_session_id} || $req->id;
 
-    # Session creation timestamp
-    my $time = $req->{sessionInfo}->{_utime} || time();
-
     # 1. LOGIN
     if ( $target eq $cas_login ) {
 
         $self->logger->debug("URL $url detected as an CAS LOGIN URL");
 
-        # GET parameters
-        my $service = $self->p->getHiddenFormValue( $req, 'service' )
-          || $req->param('service');
+        my $cas_request = {};
+
+        foreach my $param (qw/service renew gateway/) {
+            $cas_request->{$param} =
+                 $self->p->getHiddenFormValue( $req, $param )
+              || $req->param($param);
+        }
+
+        my $h = $self->p->processHook( $req, 'casGotRequest', $cas_request );
+        return $h if ( $h != PE_OK );
+
+        my $service = $cas_request->{service};
         $service = '' if ( $self->p->checkXSSAttack( 'service', $service ) );
-        my $renew = $self->p->getHiddenFormValue( $req, 'renew' )
-          || $req->param('renew');
-        my $gateway = $self->p->getHiddenFormValue( $req, 'gateway' )
-          || $req->param('gateway');
+        my $renew   = $cas_request->{renew};
+        my $gateway = $cas_request->{gateway};
+
         my $casServiceTicket;
 
         # If no service defined, exit
@@ -168,11 +201,13 @@ sub run {
             return PE_ERROR;
         }
         my ( $host, $uri ) = ( $1, $2 );
-        my $app = $self->casAppList->{$host};
+        my $app = $self->getCasApp($service);
 
-        my $spAuthnLevel =
-          $self->conf->{casAppMetaDataOptions}->{$app}
-          ->{casAppMetaDataOptionsAuthnLevel} || 0;
+        my $spAuthnLevel = 0;
+        if ($app) {
+            $spAuthnLevel =
+              $self->casAppList->{$app}->{casAppMetaDataOptionsAuthnLevel} || 0;
+        }
 
         # Renew
         if (    $renew
@@ -272,14 +307,26 @@ sub run {
             $self->logger->debug(
                 "Create a CAS service ticket for service $service");
 
+            my $_utime =
+              $self->conf->{casTicketExpiration}
+              ? (
+                time +
+                  $self->conf->{casTicketExpiration} -
+                  $self->conf->{timeout} )
+              : ( $req->{sessionInfo}->{_utime} || time() );
+
             my $Sinfos;
             $Sinfos->{type}    = 'casService';
             $Sinfos->{service} = $service;
             $Sinfos->{renew}   = $casRenewFlag;
             $Sinfos->{_cas_id} = $session_id;
-            $Sinfos->{_utime}  = $time;
+            $Sinfos->{_utime}  = $_utime;
             $Sinfos->{_casApp} = $app;
 
+            my $h = $self->p->processHook( $req, 'casGenerateServiceTicket',
+                $cas_request, $app, $Sinfos );
+            return $h if ( $h != PE_OK );
+
             my $casServiceSession = $self->getCasSession( undef, $Sinfos );
 
             unless ($casServiceSession) {
@@ -295,8 +342,9 @@ sub run {
         }
 
         # Redirect to service
-        my $service_url = $service;
-        $service_url .= ( $service =~ /\?/ ? '&' : '?' )
+        # cas_request may have been modified by hook
+        my $service_url = $cas_request->{service};
+        $service_url .= ( $service_url =~ /\?/ ? '&' : '?' )
           . build_urlencoded( ticket => $casServiceTicket );
 
         $self->logger->debug("Redirect user to $service_url");
@@ -477,6 +525,18 @@ sub validate {
         return $self->returnCasValidateError();
     }
 
+    # Make sure the token is still valid, we already compensated for
+    # different TTLs when storing _utime
+    if ( $casServiceSession->{data}->{_utime} ) {
+        if (
+            time >
+            ( $casServiceSession->{data}->{_utime} + $self->conf->{timeout} ) )
+        {
+            $self->logger->error("Session $ticket has expired");
+            return $self->returnCasValidateError();
+        }
+    }
+
     $self->logger->debug("Service ticket session $ticket found");
 
     my $service1_uri = URI->new($service);
@@ -541,6 +601,11 @@ sub validate {
 
     # Return success message
     $self->deleteCasSession($casServiceSession);
+
+    my $h =
+      $self->p->processHook( $req, 'casGenerateValidateResponse', $username );
+    return $self->returnCasValidateError() if ( $h != PE_OK );
+
     return $self->returnCasValidateSuccess( $req, $username );
 }
 
@@ -593,11 +658,16 @@ sub proxy {
             'Error in proxy session management' );
     }
 
+    my $_utime =
+      $self->conf->{casTicketExpiration}
+      ? ( time + $self->conf->{casTicketExpiration} - $self->conf->{timeout} )
+      : $casProxyGrantingSession->data->{_utime};
+
     my $Pinfos;
     $Pinfos->{type}    = 'casProxy';
     $Pinfos->{service} = $targetService;
     $Pinfos->{_cas_id} = $casProxyGrantingSession->data->{_cas_id};
-    $Pinfos->{_utime}  = $casProxyGrantingSession->data->{_utime};
+    $Pinfos->{_utime}  = $_utime;
     $Pinfos->{proxies} = $casProxyGrantingSession->data->{proxies};
 
     $casProxySession->update($Pinfos);
@@ -667,6 +737,20 @@ sub _validate2 {
         return $self->returnCasServiceValidateError( $req, 'INVALID_TICKET',
             'Ticket not found' );
     }
+
+    # Make sure the token is still valid, we already compensated for
+    # different TTLs when storing _utime
+    if ( $casServiceSession->{data}->{_utime} ) {
+        if (
+            time >
+            ( $casServiceSession->{data}->{_utime} + $self->conf->{timeout} ) )
+        {
+            $self->logger->error("$urlType ticket session $ticket has expired");
+            return $self->returnCasServiceValidateError( $req, 'INVALID_TICKET',
+                'Ticket expired' );
+        }
+    }
+
     my $app = $casServiceSession->data->{_casApp};
 
     $self->logger->debug("$urlType ticket session $ticket found");
@@ -733,7 +817,7 @@ sub _validate2 {
         $PGinfos->{type}    = 'casProxyGranting';
         $PGinfos->{service} = $service;
         $PGinfos->{_cas_id} = $casServiceSession->data->{_cas_id};
-        $PGinfos->{_utime}  = $casServiceSession->data->{_utime};
+        $PGinfos->{_utime}  = time;
         $PGinfos->{_casApp} = $app;
 
         # Trace proxies
@@ -838,6 +922,12 @@ sub _validate2 {
 
     # Return success message
     $self->deleteCasSession($casServiceSession);
+
+    my $h =
+      $self->p->processHook( $req, 'casGenerateValidateResponse', $username,
+        $attributes );
+    return $self->returnCasValidateError() if ( $h != PE_OK );
+
     return $self->returnCasServiceValidateSuccess( $req, $username,
         $casProxyGrantingTicketIOU, $proxies, $attributes );
 }
@@ -849,10 +939,8 @@ sub getUsernameForApp {
 
     my $username_attribute =
       (       $app
-          and $self->conf->{casAppMetaDataOptions}->{$app}
-          ->{casAppMetaDataOptionsUserAttribute} )
-      ? $self->conf->{casAppMetaDataOptions}->{$app}
-      ->{casAppMetaDataOptionsUserAttribute}
+          and $self->casAppList->{$app}->{casAppMetaDataOptionsUserAttribute} )
+      ? $self->casAppList->{$app}->{casAppMetaDataOptionsUserAttribute}
       : (    $self->conf->{casAttr}
           || $self->conf->{whatToTrace} );
 
