@@ -18,11 +18,12 @@ use Lemonldap::NG::Common::JWT
   qw(getAccessTokenSessionId getJWTPayload getJWTHeader getJWTSignature getJWTSignedData);
 use MIME::Base64
   qw/encode_base64 decode_base64 encode_base64url decode_base64url/;
+use Scalar::Util qw/looks_like_number/;
 use Mouse;
 
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_REDIRECT);
 
-our $VERSION = '2.0.14';
+our $VERSION = '2.0.15';
 
 # OpenID Connect standard claims
 use constant PROFILE => [
@@ -41,6 +42,7 @@ use constant OIDC_SCOPES => [qw/openid profile email address phone/];
 has oidcOPList   => ( is => 'rw', default => sub { {} }, );
 has oidcRPList   => ( is => 'rw', default => sub { {} }, );
 has rpAttributes => ( is => 'rw', default => sub { {} }, );
+has opRules      => ( is => 'rw', default => sub { {} } );
 has spRules      => ( is => 'rw', default => sub { {} } );
 has spMacros     => ( is => 'rw', default => sub { {} } );
 has spScopeRules => ( is => 'rw', default => sub { {} } );
@@ -85,11 +87,31 @@ sub loadOPs {
 
     # Extract JSON data
     foreach ( keys %{ $self->conf->{oidcOPMetaDataJSON} } ) {
-        $self->oidcOPList->{$_}->{conf} =
+        my $op_conf =
           $self->decodeJSON( $self->conf->{oidcOPMetaDataJSON}->{$_} );
-        $self->oidcOPList->{$_}->{jwks} =
-          $self->decodeJSON( $self->conf->{oidcOPMetaDataJWKS}->{$_} );
+        if ($op_conf) {
+            $self->oidcOPList->{$_}->{conf} = $op_conf;
+            $self->oidcOPList->{$_}->{jwks} =
+              $self->decodeJSON( $self->conf->{oidcOPMetaDataJWKS}->{$_} );
+        }
+        else {
+            $self->logger->warn("Could not parse OIDC metadata for $_");
+        }
     }
+
+    # Set rule
+    foreach ( keys %{ $self->conf->{oidcOPMetaDataOptions} } ) {
+        my $cond = $self->conf->{oidcOPMetaDataOptions}->{$_}
+          ->{oidcOPMetaDataOptionsResolutionRule};
+        if ( length $cond ) {
+            my $rule_sub =
+              $self->p->buildRule( $cond, "OIDC provider resolution" );
+            if ($rule_sub) {
+                $self->opRules->{$_} = $rule_sub;
+            }
+        }
+    }
+
     return 1;
 }
 
@@ -304,6 +326,14 @@ sub buildAuthorizationCodeAuthnRequest {
 
     my $authorize_uri =
       $self->oidcOPList->{$op}->{conf}->{authorization_endpoint};
+
+    unless ($authorize_uri) {
+        $self->logger->error(
+            "Could not build Authorize request: no
+            'authorization_endpoint'" . " in JSON metadata for OP $op"
+        );
+        return undef;
+    }
     my $client_id =
       $self->conf->{oidcOPMetaDataOptions}->{$op}
       ->{oidcOPMetaDataOptionsClientID};
@@ -333,10 +363,7 @@ sub buildAuthorizationCodeAuthnRequest {
     my $nonce;
     $nonce = $self->ott->createToken if ($use_nonce);
 
-    my $authn_uri =
-        $authorize_uri
-      . ( $authorize_uri =~ /\?/ ? '&' : '?' )
-      . build_urlencoded(
+    my $authorize_request_params = {
         response_type => $response_type,
         client_id     => $client_id,
         scope         => $scope,
@@ -348,7 +375,19 @@ sub buildAuthorizationCodeAuthnRequest {
         ( $max_age            ? ( max_age    => $max_age )    : () ),
         ( defined $ui_locales ? ( ui_locales => $ui_locales ) : () ),
         ( defined $acr_values ? ( acr_values => $acr_values ) : () )
-      );
+    };
+
+    # Call oidcGenerateAuthenticationRequest
+    my $h = $self->p->processHook(
+        $req, 'oidcGenerateAuthenticationRequest',
+        $op,  $authorize_request_params
+    );
+    return if ( $h != PE_OK );
+
+    my $authn_uri =
+        $authorize_uri
+      . ( $authorize_uri =~ /\?/ ? '&' : '?' )
+      . build_urlencoded(%$authorize_request_params);
 
     $self->logger->debug(
         "OpenIDConnect Authorization Code Flow Authn Request: $authn_uri");
@@ -441,13 +480,10 @@ sub buildHybridAuthnResponse {
     return $response_url;
 }
 
-# Get Token response with authorization code
-# @param op OpenIP Provider configuration key
-# @param code Code
-# @param auth_method Authentication Method
-# return String Token response decoded content
-sub getAuthorizationCodeAccessToken {
-    my ( $self, $req, $op, $code, $auth_method ) = @_;
+sub getAccessTokenFromTokenEndpoint {
+    my ( $self, $req, $op, $grant_type, $grant_options ) = @_;
+
+    $grant_options ||= {};
 
     my $client_id =
       $self->conf->{oidcOPMetaDataOptions}->{$op}
@@ -455,13 +491,25 @@ sub getAuthorizationCodeAccessToken {
     my $client_secret =
       $self->conf->{oidcOPMetaDataOptions}->{$op}
       ->{oidcOPMetaDataOptionsClientSecret};
-    my $redirect_uri = $self->getCallbackUri($req);
     my $access_token_uri =
       $self->oidcOPList->{$op}->{conf}->{token_endpoint};
-    my $grant_type = "authorization_code";
+
+    unless ($access_token_uri) {
+        $self->logger->error(
+            "Could not build Token request: no
+            'token_endpoint'" . " in JSON metadata for OP $op"
+        );
+        return 0;
+    }
+
+    my $auth_method =
+      $self->conf->{oidcOPMetaDataOptions}->{$op}
+      ->{oidcOPMetaDataOptionsTokenEndpointAuthMethod}
+      || 'client_secret_post';
 
     unless ( $auth_method =~ /^client_secret_(basic|post)$/o ) {
-        $self->logger->error("Bad authentication method on token endpoint");
+        $self->logger->error(
+            "Bad authentication method on token endpoint for OP $op");
         return 0;
     }
 
@@ -469,32 +517,29 @@ sub getAuthorizationCodeAccessToken {
         "Using auth method $auth_method to token endpoint $access_token_uri");
 
     my $response;
+    my $token_request_params = {
+        grant_type => $grant_type,
+        %{$grant_options}
+    };
+
+    # Call oidcGenerateTokenRequest
+    my $h = $self->p->processHook( $req, 'oidcGenerateTokenRequest',
+        $op, $token_request_params );
+    return 0 if ( $h != PE_OK );
 
     if ( $auth_method eq "client_secret_basic" ) {
-        my $form = {
-            code         => $code,
-            redirect_uri => $redirect_uri,
-            grant_type   => $grant_type
-        };
-
         $response = $self->ua->post(
-            $access_token_uri, $form,
+            $access_token_uri, $token_request_params,
             "Authorization" => "Basic "
               . encode_base64( "$client_id:$client_secret", '' ),
             "Content-Type" => 'application/x-www-form-urlencoded',
         );
     }
-
     elsif ( $auth_method eq "client_secret_post" ) {
-        my $form = {
-            code          => $code,
-            client_id     => $client_id,
-            client_secret => $client_secret,
-            redirect_uri  => $redirect_uri,
-            grant_type    => $grant_type
-        };
+        $token_request_params->{client_id}     = $client_id;
+        $token_request_params->{client_secret} = $client_secret;
 
-        $response = $self->ua->post( $access_token_uri, $form,
+        $response = $self->ua->post( $access_token_uri, $token_request_params,
             "Content-Type" => 'application/x-www-form-urlencoded' );
     }
     else {
@@ -502,14 +547,28 @@ sub getAuthorizationCodeAccessToken {
     }
 
     if ( $response->is_error ) {
-        $self->logger->error(
-            "Bad authorization response: " . $response->message );
+        $self->logger->error( "Bad token response: " . $response->message );
         $self->logger->debug( $response->content );
         return 0;
     }
     return $response->decoded_content;
 }
 
+# Get Token response with authorization code
+# @param op OpenIP Provider configuration key
+# @param code Code
+# @param auth_method Authentication Method (optional)
+# return String Token response decoded content
+sub getAuthorizationCodeAccessToken {
+    my ( $self, $req, $op, $code ) = @_;
+
+    my $redirect_uri = $self->getCallbackUri($req);
+
+    return $self->getAccessTokenFromTokenEndpoint( $req, $op,
+        "authorization_code",
+        { code => $code, redirect_uri => $redirect_uri } );
+}
+
 # Check validity of Token Response
 # return boolean 1 if the response is valid, 0 else
 sub checkTokenResponseValidity {
@@ -645,6 +704,134 @@ sub checkIDTokenValidity {
     return 1;
 }
 
+# Returns the current OP and a valid Access token
+sub getUserInfoParams {
+    my ( $self, $req ) = @_;
+
+    my $op = $req->data->{_oidcOPCurrent};
+
+    if ($op) {
+
+        # We are in the middle of an auth process,
+        # access token has just been fetched already
+        my $access_token = $req->data->{access_token};
+        return ( $op, $access_token );
+    }
+    else {
+        # Get OP and access token from existing session (refresh)
+        return $self->getUserInfoParamsFromSession($req);
+    }
+}
+
+sub getUserInfoParamsFromSession {
+    my ( $self, $req ) = @_;
+    my $op = $req->userData->{_oidc_OP};
+
+    # Save current OP, we will need it for setSessionInfo & friends
+    $req->data->{_oidcOPCurrent} = $op;
+
+    if ($op) {
+        my $access_token     = $req->userData->{_oidc_access_token};
+        my $access_token_eol = $req->userData->{_oidc_access_token_eol};
+        if ($access_token_eol) {
+            return $self->refreshAccessTokenIfExpired( $req, $op );
+        }
+        else {
+            # We don't know the TTL for this access token,
+            # so we can only hope that it works
+            return ( $op, $access_token );
+        }
+    }
+    else {
+        $self->logger->warn("No OP found in session");
+        return ( $op, undef );
+    }
+}
+
+sub refreshAccessTokenIfExpired {
+    my ( $self, $req, $op, $session ) = @_;
+
+    # Handle unauthenticated OIDC calls
+    my $data = $session ? $session->data : $req->userData;
+
+    my $access_token     = $data->{_oidc_access_token};
+    my $access_token_eol = $data->{_oidc_access_token_eol};
+    if ( time < $access_token_eol ) {
+
+        # Access Token is still valid, return it
+        return ( $op, $access_token );
+    }
+    else {
+        # Refresh Access Token
+        return ( $op, $self->refreshAccessToken( $req, $op, $session ) );
+    }
+}
+
+sub refreshAccessToken {
+    my ( $self, $req, $op, $session ) = @_;
+
+    # Handle unauthenticated OIDC calls
+    my $data       = $session ? $session->data : $req->userData;
+    my $session_id = $session ? $session->id   : $req->id;
+
+    my $refresh_token = $data->{_oidc_refresh_token};
+
+    if ($refresh_token) {
+
+        my $content =
+          $self->getAccessTokenFromTokenEndpoint( $req, $op, 'refresh_token',
+            { refresh_token => $refresh_token } );
+
+        if ($content) {
+            my $token_response = $self->decodeTokenResponse($content);
+            if ($token_response) {
+
+                my $access_token  = $token_response->{access_token};
+                my $expires_in    = $token_response->{expires_in};
+                my $refresh_token = $token_response->{refresh_token};
+
+                undef $expires_in unless looks_like_number($expires_in);
+
+                $self->logger->debug("Access token: $access_token");
+                $self->logger->debug( "Access token expires in: "
+                      . ( $expires_in || "<unknown>" ) );
+                $self->logger->debug(
+                    "Refresh token: " . ( $refresh_token || "<none>" ) );
+
+                my $updateSession;
+
+                # Remember tokens
+                $updateSession->{_oidc_access_token}  = $access_token;
+                $updateSession->{_oidc_refresh_token} = $refresh_token
+                  if $refresh_token;
+
+                # If access token TTL is given save expiration date
+                # (with security margin)
+                if ($expires_in) {
+                    $updateSession->{_oidc_access_token_eol} =
+                      time + ( $expires_in * 0.9 );
+                }
+
+                $self->p->updateSession( $req, $updateSession, $session_id );
+
+                return ($access_token);
+            }
+            else {
+                $self->logger->warn("Could not decode Token Response for $op");
+                return undef;
+            }
+        }
+        else {
+            $self->logger->warn("Could not fetch new Access Token for $op");
+            return undef;
+        }
+    }
+    else {
+        $self->logger->warn("No Refresh Token was found for $op");
+        return undef;
+    }
+}
+
 # Get UserInfo response
 # return String UserInfo response decoded content
 sub getUserInfo {
@@ -806,15 +993,8 @@ sub makeJWT {
         scope     => $scope,                      # Scope
     };
 
-    my $claims;
-    if ( ref($sessionInfo) eq "HASH" ) {
-        $claims = $self->buildUserInfoResponseFromData( $req, $scope,
-            $rp, $sessionInfo );
-    }
-    else {
-        $claims =
-          $self->buildUserInfoResponseFromId( $req, $scope, $rp, $sessionInfo );
-    }
+    my $claims =
+      $self->buildUserInfoResponseFromData( $req, $scope, $rp, $sessionInfo );
 
     # Release claims, or only sub
     if ( $self->conf->{oidcRPMetaDataOptions}->{$rp}
@@ -903,7 +1083,7 @@ sub getRefreshToken {
 
 sub updateRefreshToken {
     my $self = shift;
-    return $self->updateToken($@);
+    return $self->updateToken(@_);
 }
 
 sub updateToken {
@@ -1440,7 +1620,7 @@ sub getEndPointAuthenticationCredentials {
         $self->logger->error("Bad authentication header: $@") if ($@);
 
         # Using multiple methods is an error
-        if ( $req->param('client_id') ) {
+        if ( $req->param('client_id') and $req->param('client_secret') ) {
             $self->logger->error("Multiple client authentication methods used");
             ( $client_id, $client_secret ) = ( undef, undef );
         }
@@ -1652,7 +1832,7 @@ sub buildUserInfoResponseFromData {
     }
 
     my $h = $self->p->processHook( $req, 'oidcGenerateUserInfoResponse',
-        $userinfo_response, $rp );
+        $userinfo_response, $rp, $data );
     return {} if ( $h != PE_OK );
 
     return $userinfo_response;
@@ -2068,18 +2248,11 @@ sub getUserIDForRP {
       ->{oidcRPMetaDataOptionsUserIDAttr}
       || $self->conf->{whatToTrace};
 
-    my $user_id;
-
     # If the main attribute is a SP macro, resolve it
     # else, get it directly from session data
-    if ( $self->spMacros->{$rp}->{$user_id_attribute} ) {
-        $user_id =
-          $self->spMacros->{$rp}->{$user_id_attribute}->( $req, $data );
-    }
-    else {
-        $user_id = $data->{$user_id_attribute};
-    }
-    return $user_id;
+    return $self->spMacros->{$rp}->{$user_id_attribute}
+      ? $self->spMacros->{$rp}->{$user_id_attribute}->( $req, $data )
+      : $data->{$user_id_attribute};
 }
 
 1;
