@@ -9,7 +9,7 @@
 #
 package Lemonldap::NG::Portal::Main::Run;
 
-our $VERSION = '2.0.15';
+our $VERSION = '2.0.15.1';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -876,7 +876,7 @@ sub _dump {
 sub getTrOver {
     my ( $self, $req, $templateDir ) = @_;
 
-    my $templateDir //= $self->conf->{templateDir} . '/' . $self->getSkin($req);
+    $templateDir //= $self->conf->{templateDir} . '/' . $self->getSkin($req);
 
     unless ( $self->trOverCache->{$templateDir} ) {
 
