@@ -9,7 +9,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_BADCREDENTIALS
 );
 
-our $VERSION = '2.0.14';
+our $VERSION = '2.0.15';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Plugin
@@ -159,10 +159,6 @@ sub display {
         $self->_createArray( $req, $attrs, $req->userData ) );
 
     my $params = {
-        PORTAL     => $self->conf->{portal},
-        MAIN_LOGO  => $self->conf->{portalMainLogo},
-        SKIN       => $self->p->getSkin($req),
-        LANGS      => $self->conf->{showLanguages},
         MSG        => 'checkUser' . $self->merged,
         ALERTE     => ( $self->merged ? 'alert-warning' : 'alert-info' ),
         LOGIN      => $req->{userData}->{ $self->conf->{whatToTrace} },
@@ -210,14 +206,10 @@ sub check {
         }
 
         my $params = {
-            PORTAL    => $self->conf->{portal},
-            MAIN_LOGO => $self->conf->{portalMainLogo},
-            SKIN      => $self->p->getSkin($req),
-            LANGS     => $self->conf->{showLanguages},
-            MSG       => "PE$msg",
-            ALERTE    => 'alert-warning',
-            LOGIN     => '',
-            TOKEN     => $token,
+            MSG    => "PE$msg",
+            ALERTE => 'alert-warning',
+            LOGIN  => '',
+            TOKEN  => $token,
         };
         return $self->p->sendJSONresponse( $req, $params )
           if $req->wantJSON && $msg;
@@ -241,14 +233,10 @@ sub check {
             $req,
             'checkuser',
             params => {
-                PORTAL    => $self->conf->{portal},
-                MAIN_LOGO => $self->conf->{portalMainLogo},
-                SKIN      => $self->p->getSkin($req),
-                LANGS     => $self->conf->{showLanguages},
-                MSG       => 'PE' . PE_MALFORMEDUSER,
-                ALERTE    => 'alert-warning',
-                LOGIN     => '',
-                TOKEN     => (
+                MSG    => 'PE' . PE_MALFORMEDUSER,
+                ALERTE => 'alert-warning',
+                LOGIN  => '',
+                TOKEN  => (
                       $self->ottRule->( $req, {} )
                     ? $self->ott->createToken()
                     : ''
@@ -409,10 +397,6 @@ sub check {
 
     # TODO:
     my $params = {
-        PORTAL      => $self->conf->{portal},
-        MAIN_LOGO   => $self->conf->{portalMainLogo},
-        SKIN        => $self->p->getSkin($req),
-        LANGS       => $self->conf->{showLanguages},
         MSG         => $msg,
         ALERTE      => ( $msg eq 'checkUser' ? 'alert-info' : 'alert-warning' ),
         LOGIN       => $user,
