@@ -22,7 +22,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_MAILCONFIRMATION_ALREADY_SENT
 );
 
-our $VERSION = '2.0.13';
+our $VERSION = '2.0.15';
 
 extends qw(
   Lemonldap::NG::Portal::Lib::SMTP
@@ -76,7 +76,7 @@ sub init {
 
     # Initialize Captcha if needed
     if ( $self->conf->{captcha_register_enabled} ) {
-        $self->captcha( $self->p->loadModule('::Lib::Captcha') ) or return 0;
+        $self->captcha(1);
     }
 
     # Initialize form token if needed (captcha provides also a token)
@@ -168,44 +168,28 @@ sub _register {
             !$self->getRegisterSession( $req->data->{registerInfo}->{mail} ) )
         {
 
-            # Check if token exists
-            my $token;
-            if ( $self->ottRule->( $req, {} ) or $self->captcha ) {
-                $token = $req->param('token');
+            # Captcha for register form
+            if ( $self->captcha ) {
+                my $result = $self->p->_captcha->check_captcha($req);
+                if ($result) {
+                    $self->logger->debug("Captcha code verified");
+                }
+                else {
+                    $self->setSecurity($req);
+                    $self->userLogger->warn("Captcha failed");
+                    return PE_CAPTCHAERROR;
+                }
+            }
+            elsif ( $self->ottRule->( $req, {} ) ) {
+                my $token = $req->param('token');
                 unless ($token) {
                     $self->setSecurity($req);
                     $self->userLogger->warn('Register try without token');
                     return PE_NOTOKEN;
                 }
-            }
-
-            # Captcha for register form
-            if ( $self->captcha ) {
-                my $captcha = $req->param('captcha');
-
-                unless ($captcha) {
-                    $self->userLogger->warn(
-                        'Register try with captcha not filled');
-
-                    # Set captcha or token
-                    $self->setSecurity($req);
-                    return PE_CAPTCHAEMPTY;
-                }
-
-                # Check captcha
-                unless ( $self->captcha->validateCaptcha( $token, $captcha ) ) {
-                    $self->userLogger->info('Captcha failed: wrong code');
-
-                    # Set captcha or token
-                    $self->setSecurity($req);
-                    return PE_CAPTCHAERROR;
-                }
-                $self->logger->debug("Captcha code verified");
-            }
-            elsif ( $self->ottRule->( $req, {} ) ) {
                 unless ( $self->ott->getToken($token) ) {
                     $self->setSecurity($req);
-                    $self->userLogger->notice(
+                    $self->userLogger->warn(
                         'Register try with expired/bad token');
                     return PE_TOKENEXPIRED;
                 }
@@ -445,7 +429,12 @@ sub display {
         STARTMAILDATE   => $req->data->{startMailDate},
         STARTMAILTIME   => $req->data->{startMailTime},
         MAILALREADYSENT => $req->data->{mail_already_sent},
-        MAIL            => $self->p->checkXSSAttack(
+        (
+            $req->data->{customScript}
+            ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+            : ()
+        ),
+        MAIL => $self->p->checkXSSAttack(
             'mail', $req->data->{registerInfo}->{mail}
           ) ? ""
         : $req->data->{registerInfo}->{mail},
@@ -483,14 +472,19 @@ sub display {
     }
 
     # Display captcha if it's enabled
-    if ( $req->captcha ) {
-        $templateParams{CAPTCHA_SRC}  = $req->captcha;
-        $templateParams{CAPTCHA_SIZE} = $self->conf->{captcha_size} || 6;
+    if ( $req->captchaHtml ) {
+        $templateParams{CAPTCHA_HTML} = $req->captchaHtml;
     }
     if ( $req->token ) {
         $templateParams{TOKEN} = $req->token;
     }
 
+    # DEPRECATED: This is only used for compatibility with existing templates
+    if ( $req->captcha ) {
+        $templateParams{CAPTCHA_SRC}  = $req->captcha;
+        $templateParams{CAPTCHA_SIZE} = $self->conf->{captcha_size};
+    }
+
     if ( $req->error == PE_REGISTERALREADYEXISTS ) {
         %templateParams = (
             %templateParams,
@@ -560,7 +554,7 @@ sub display {
 sub setSecurity {
     my ( $self, $req ) = @_;
     if ( $self->captcha ) {
-        $self->captcha->setCaptcha($req);
+        $self->p->_captcha->init_captcha($req);
     }
     elsif ( $self->ottRule->( $req, {} ) ) {
         $self->ott->setToken($req);
